@@ -113,8 +113,22 @@ func (cli *Client) GetServer() (*config.Global, error) {
 	}, nil
 }
 
-func (cli *Client) getNeighbor(name string, afi int, vrf string) ([]*config.Neighbor, error) {
-	ret, err := cli.cli.GetNeighbor(context.Background(), &api.GetNeighborRequest{EnableAdvertised: name != ""})
+func (cli *Client) EnableZebra(c *config.Zebra) error {
+	req := &api.EnableZebraRequest{
+		Url:                  c.Config.Url,
+		Version:              uint32(c.Config.Version),
+		NexthopTriggerEnable: c.Config.NexthopTriggerEnable,
+		NexthopTriggerDelay:  uint32(c.Config.NexthopTriggerDelay),
+	}
+	for _, t := range c.Config.RedistributeRouteTypeList {
+		req.RouteTypes = append(req.RouteTypes, string(t))
+	}
+	_, err := cli.cli.EnableZebra(context.Background(), req)
+	return err
+}
+
+func (cli *Client) getNeighbor(name string, afi int, vrf string, enableAdvertised bool) ([]*config.Neighbor, error) {
+	ret, err := cli.cli.GetNeighbor(context.Background(), &api.GetNeighborRequest{EnableAdvertised: enableAdvertised, Address: name})
 	if err != nil {
 		return nil, err
 	}
@@ -122,14 +136,14 @@ func (cli *Client) getNeighbor(name string, afi int, vrf string) ([]*config.Neig
 	neighbors := make([]*config.Neighbor, 0, len(ret.Peers))
 
 	for _, p := range ret.Peers {
-		if name != "" && name != p.Conf.NeighborAddress {
+		if name != "" && name != p.Info.NeighborAddress && name != p.Conf.NeighborInterface {
 			continue
 		}
 		if vrf != "" && name != p.Conf.Vrf {
 			continue
 		}
 		if afi > 0 {
-			v6 := net.ParseIP(p.Conf.NeighborAddress).To4() == nil
+			v6 := net.ParseIP(p.Info.NeighborAddress).To4() == nil
 			if afi == bgp.AFI_IP && v6 || afi == bgp.AFI_IP6 && !v6 {
 				continue
 			}
@@ -144,19 +158,23 @@ func (cli *Client) getNeighbor(name string, afi int, vrf string) ([]*config.Neig
 }
 
 func (cli *Client) ListNeighbor() ([]*config.Neighbor, error) {
-	return cli.getNeighbor("", 0, "")
+	return cli.getNeighbor("", 0, "", false)
 }
 
 func (cli *Client) ListNeighborByTransport(afi int) ([]*config.Neighbor, error) {
-	return cli.getNeighbor("", afi, "")
+	return cli.getNeighbor("", afi, "", false)
 }
 
 func (cli *Client) ListNeighborByVRF(vrf string) ([]*config.Neighbor, error) {
-	return cli.getNeighbor("", 0, vrf)
+	return cli.getNeighbor("", 0, vrf, false)
 }
 
-func (cli *Client) GetNeighbor(name string) (*config.Neighbor, error) {
-	ns, err := cli.getNeighbor(name, 0, "")
+func (cli *Client) GetNeighbor(name string, options ...bool) (*config.Neighbor, error) {
+	enableAdvertised := false
+	if len(options) > 0 && options[0] {
+		enableAdvertised = true
+	}
+	ns, err := cli.getNeighbor(name, 0, "", enableAdvertised)
 	if err != nil {
 		return nil, err
 	}
@@ -181,13 +199,13 @@ func (cli *Client) DeleteNeighbor(c *config.Neighbor) error {
 //func (cli *Client) UpdateNeighbor(c *config.Neighbor) (bool, error) {
 //}
 
-func (cli *Client) ShutdownNeighbor(addr string) error {
-	_, err := cli.cli.ShutdownNeighbor(context.Background(), &api.ShutdownNeighborRequest{Address: addr})
+func (cli *Client) ShutdownNeighbor(addr, communication string) error {
+	_, err := cli.cli.ShutdownNeighbor(context.Background(), &api.ShutdownNeighborRequest{Address: addr, Communication: communication})
 	return err
 }
 
-func (cli *Client) ResetNeighbor(addr string) error {
-	_, err := cli.cli.ResetNeighbor(context.Background(), &api.ResetNeighborRequest{Address: addr})
+func (cli *Client) ResetNeighbor(addr, communication string) error {
+	_, err := cli.cli.ResetNeighbor(context.Background(), &api.ResetNeighborRequest{Address: addr, Communication: communication})
 	return err
 }
 
@@ -196,8 +214,8 @@ func (cli *Client) EnableNeighbor(addr string) error {
 	return err
 }
 
-func (cli *Client) DisableNeighbor(addr string) error {
-	_, err := cli.cli.DisableNeighbor(context.Background(), &api.DisableNeighborRequest{Address: addr})
+func (cli *Client) DisableNeighbor(addr, communication string) error {
+	_, err := cli.cli.DisableNeighbor(context.Background(), &api.DisableNeighborRequest{Address: addr, Communication: communication})
 	return err
 }
 
@@ -517,8 +535,11 @@ func (cli *Client) DeleteVRF(name string) error {
 	return err
 }
 
-func (cli *Client) GetDefinedSet(typ table.DefinedType) ([]table.DefinedSet, error) {
-	ret, err := cli.cli.GetDefinedSet(context.Background(), &api.GetDefinedSetRequest{Type: api.DefinedType(typ)})
+func (cli *Client) getDefinedSet(typ table.DefinedType, name string) ([]table.DefinedSet, error) {
+	ret, err := cli.cli.GetDefinedSet(context.Background(), &api.GetDefinedSetRequest{
+		Type: api.DefinedType(typ),
+		Name: name,
+	})
 	if err != nil {
 		return nil, err
 	}
@@ -533,6 +554,23 @@ func (cli *Client) GetDefinedSet(typ table.DefinedType) ([]table.DefinedSet, err
 	return ds, nil
 }
 
+func (cli *Client) GetDefinedSet(typ table.DefinedType) ([]table.DefinedSet, error) {
+	return cli.getDefinedSet(typ, "")
+}
+
+func (cli *Client) GetDefinedSetByName(typ table.DefinedType, name string) (table.DefinedSet, error) {
+	sets, err := cli.getDefinedSet(typ, name)
+	if err != nil {
+		return nil, err
+	}
+	if len(sets) == 0 {
+		return nil, fmt.Errorf("not found defined set: %s", name)
+	} else if len(sets) > 1 {
+		return nil, fmt.Errorf("invalid response for GetDefinedSetByName")
+	}
+	return sets[0], nil
+}
+
 func (cli *Client) AddDefinedSet(d table.DefinedSet) error {
 	a, err := api.NewAPIDefinedSetFromTableStruct(d)
 	if err != nil {
@@ -900,10 +938,13 @@ func (c *MonitorRIBClient) Recv() (*table.Destination, error) {
 	return d.ToNativeDestination()
 }
 
-func (cli *Client) MonitorRIB(family bgp.RouteFamily) (*MonitorRIBClient, error) {
-	stream, err := cli.cli.MonitorRib(context.Background(), &api.Table{
-		Type:   api.Resource_GLOBAL,
-		Family: uint32(family),
+func (cli *Client) MonitorRIB(family bgp.RouteFamily, current bool) (*MonitorRIBClient, error) {
+	stream, err := cli.cli.MonitorRib(context.Background(), &api.MonitorRibRequest{
+		Table: &api.Table{
+			Type:   api.Resource_GLOBAL,
+			Family: uint32(family),
+		},
+		Current: current,
 	})
 	if err != nil {
 		return nil, err
@@ -911,11 +952,14 @@ func (cli *Client) MonitorRIB(family bgp.RouteFamily) (*MonitorRIBClient, error)
 	return &MonitorRIBClient{stream}, nil
 }
 
-func (cli *Client) MonitorAdjRIBIn(name string, family bgp.RouteFamily) (*MonitorRIBClient, error) {
-	stream, err := cli.cli.MonitorRib(context.Background(), &api.Table{
-		Type:   api.Resource_ADJ_IN,
-		Name:   name,
-		Family: uint32(family),
+func (cli *Client) MonitorAdjRIBIn(name string, family bgp.RouteFamily, current bool) (*MonitorRIBClient, error) {
+	stream, err := cli.cli.MonitorRib(context.Background(), &api.MonitorRibRequest{
+		Table: &api.Table{
+			Type:   api.Resource_ADJ_IN,
+			Name:   name,
+			Family: uint32(family),
+		},
+		Current: current,
 	})
 	if err != nil {
 		return nil, err
