@@ -18,12 +18,28 @@ package config
 import (
 	"fmt"
 	"net"
+	"path/filepath"
 	"regexp"
 	"strconv"
 
 	"github.com/osrg/gobgp/packet/bgp"
 )
 
+// Returns config file type by retrieving extension from the given path.
+// If no corresponding type found, returns the given def as the default value.
+func detectConfigFileType(path, def string) string {
+	switch ext := filepath.Ext(path); ext {
+	case ".toml":
+		return "toml"
+	case ".yaml", ".yml":
+		return "yaml"
+	case ".json":
+		return "json"
+	default:
+		return def
+	}
+}
+
 func IsConfederationMember(g *Global, p *Neighbor) bool {
 	if p.Config.PeerAs != g.Config.As {
 		for _, member := range g.Confederation.Config.MemberAsList {
@@ -124,3 +140,14 @@ func ParseMaskLength(prefix, mask string) (int, int, error) {
 	}
 	return min, max, nil
 }
+
+func ExtractNeighborAddress(c *Neighbor) (string, error) {
+	addr := c.State.NeighborAddress
+	if addr == "" {
+		addr = c.Config.NeighborAddress
+		if addr == "" {
+			return "", fmt.Errorf("NeighborAddress is not configured")
+		}
+	}
+	return addr, nil
+}
