@@ -24,24 +24,32 @@ import (
 	"strconv"
 	"time"
 
-	log "github.com/Sirupsen/logrus"
 	"github.com/armon/go-radix"
 	"github.com/osrg/gobgp/config"
 	"github.com/osrg/gobgp/packet/bgp"
 	"github.com/osrg/gobgp/packet/rtr"
 	"github.com/osrg/gobgp/table"
-	"gopkg.in/tomb.v2"
+	log "github.com/sirupsen/logrus"
+	"golang.org/x/net/context"
+)
+
+const (
+	CONNECT_RETRY_INTERVAL = 30
 )
 
 func before(a, b uint32) bool {
 	return int32(a-b) < 0
 }
 
-type roaBucket struct {
+type RoaBucket struct {
 	Prefix  *table.IPPrefix
 	entries []*table.ROA
 }
 
+func (r *RoaBucket) GetEntries() []*table.ROA {
+	return r.entries
+}
+
 type roas []*table.ROA
 
 func (r roas) Len() int {
@@ -125,9 +133,7 @@ func (m *roaManager) AddServer(host string, lifetime int64) error {
 	if _, ok := m.clientMap[host]; ok {
 		return fmt.Errorf("ROA server exists %s", host)
 	}
-	client := NewRoaClient(address, port, m.eventCh, lifetime)
-	m.clientMap[host] = client
-	client.t.Go(client.tryConnect)
+	m.clientMap[host] = NewRoaClient(address, port, m.eventCh, lifetime)
 	return nil
 }
 
@@ -136,7 +142,8 @@ func (m *roaManager) DeleteServer(host string) error {
 	if !ok {
 		return fmt.Errorf("ROA server doesn't exists %s", host)
 	}
-	client.reset()
+	client.stop()
+	m.deleteAllROA(host)
 	delete(m.clientMap, host)
 	return nil
 }
@@ -145,7 +152,7 @@ func (m *roaManager) deleteAllROA(network string) {
 	for _, tree := range m.Roas {
 		deleteKeys := make([]string, 0, tree.Len())
 		tree.Walk(func(s string, v interface{}) bool {
-			b, _ := v.(*roaBucket)
+			b, _ := v.(*RoaBucket)
 			newEntries := make([]*table.ROA, 0, len(b.entries))
 			for _, r := range b.entries {
 				if r.Src != network {
@@ -181,6 +188,7 @@ func (m *roaManager) Disable(address string) error {
 		add, _, _ := net.SplitHostPort(network)
 		if add == address {
 			client.reset()
+			m.deleteAllROA(add)
 			return nil
 		}
 	}
@@ -188,14 +196,7 @@ func (m *roaManager) Disable(address string) error {
 }
 
 func (m *roaManager) Reset(address string) error {
-	for network, client := range m.clientMap {
-		add, _, _ := net.SplitHostPort(network)
-		if add == address {
-			client.reset()
-			return nil
-		}
-	}
-	return fmt.Errorf("ROA server not found %s", address)
+	return m.Disable(address)
 }
 
 func (m *roaManager) SoftReset(address string) error {
@@ -239,16 +240,14 @@ func (m *roaManager) HandleROAEvent(ev *ROAEvent) {
 		client.pendingROAs = make([]*table.ROA, 0)
 		client.state.RpkiMessages = config.RpkiMessages{}
 		client.conn = nil
-		client.t = tomb.Tomb{}
-		client.t.Go(client.tryConnect)
+		go client.tryConnect()
 		client.timer = time.AfterFunc(time.Duration(client.lifetime)*time.Second, client.lifetimeout)
 		client.oldSessionID = client.sessionID
 	case CONNECTED:
 		log.WithFields(log.Fields{"Topic": "rpki"}).Infof("ROA server %s is connected", ev.Src)
 		client.conn = ev.conn
 		client.state.Uptime = time.Now().Unix()
-		client.t = tomb.Tomb{}
-		client.t.Go(client.established)
+		go client.established()
 	case RTR:
 		m.handleRTRMsg(client, &client.state, ev.Data)
 	case LIFETIMEOUT:
@@ -280,7 +279,7 @@ func (m *roaManager) deleteROA(roa *table.ROA) {
 	tree, key := m.roa2tree(roa)
 	b, _ := tree.Get(key)
 	if b != nil {
-		bucket := b.(*roaBucket)
+		bucket := b.(*RoaBucket)
 		newEntries := make([]*table.ROA, 0, len(bucket.entries))
 		for _, r := range bucket.entries {
 			if !r.Equal(roa) {
@@ -307,15 +306,15 @@ func (m *roaManager) deleteROA(roa *table.ROA) {
 func (m *roaManager) addROA(roa *table.ROA) {
 	tree, key := m.roa2tree(roa)
 	b, _ := tree.Get(key)
-	var bucket *roaBucket
+	var bucket *RoaBucket
 	if b == nil {
-		bucket = &roaBucket{
+		bucket = &RoaBucket{
 			Prefix:  roa.Prefix,
 			entries: make([]*table.ROA, 0),
 		}
 		tree.Insert(key, bucket)
 	} else {
-		bucket = b.(*roaBucket)
+		bucket = b.(*RoaBucket)
 		for _, r := range bucket.entries {
 			if r.Equal(roa) {
 				// we already have the same one
@@ -404,7 +403,7 @@ func (c *roaManager) GetServers() []*config.RpkiServer {
 		prefixes := make(map[string]uint32)
 
 		tree.Walk(func(s string, v interface{}) bool {
-			b, _ := v.(*roaBucket)
+			b, _ := v.(*RoaBucket)
 			tmpRecords := make(map[string]uint32)
 			for _, roa := range b.entries {
 				tmpRecords[roa.Src]++
@@ -428,15 +427,6 @@ func (c *roaManager) GetServers() []*config.RpkiServer {
 	for _, client := range c.clientMap {
 		state := &client.state
 
-		addr, port, _ := net.SplitHostPort(client.host)
-		l = append(l, &config.RpkiServer{
-			Config: config.RpkiServerConfig{
-				Address: addr,
-				Port:    func() uint32 { p, _ := strconv.Atoi(port); return uint32(p) }(),
-			},
-			State: client.state,
-		})
-
 		if client.conn == nil {
 			state.Up = false
 		} else {
@@ -453,6 +443,15 @@ func (c *roaManager) GetServers() []*config.RpkiServer {
 		state.PrefixesV4 = f(prefixesV4, client.host)
 		state.PrefixesV6 = f(prefixesV6, client.host)
 		state.SerialNumber = client.serialNumber
+
+		addr, port, _ := net.SplitHostPort(client.host)
+		l = append(l, &config.RpkiServer{
+			Config: config.RpkiServerConfig{
+				Address: addr,
+				Port:    func() uint32 { p, _ := strconv.Atoi(port); return uint32(p) }(),
+			},
+			State: client.state,
+		})
 	}
 	return l
 }
@@ -474,7 +473,7 @@ func (c *roaManager) GetRoa(family bgp.RouteFamily) ([]*table.ROA, error) {
 	for _, rf := range rfList {
 		if tree, ok := c.Roas[rf]; ok {
 			tree.Walk(func(s string, v interface{}) bool {
-				b, _ := v.(*roaBucket)
+				b, _ := v.(*RoaBucket)
 				var roaList roas
 				for _, r := range b.entries {
 					roaList = append(roaList, r)
@@ -490,7 +489,7 @@ func (c *roaManager) GetRoa(family bgp.RouteFamily) ([]*table.ROA, error) {
 	return l, nil
 }
 
-func validatePath(ownAs uint32, tree *radix.Tree, cidr string, asPath *bgp.PathAttributeAsPath) config.RpkiValidationResultType {
+func ValidatePath(ownAs uint32, tree *radix.Tree, cidr string, asPath *bgp.PathAttributeAsPath) (config.RpkiValidationResultType, *RoaBucket) {
 	var as uint32
 
 	if asPath == nil || len(asPath.Value) == 0 {
@@ -507,27 +506,32 @@ func validatePath(ownAs uint32, tree *radix.Tree, cidr string, asPath *bgp.PathA
 		case bgp.BGP_ASPATH_ATTR_TYPE_CONFED_SET, bgp.BGP_ASPATH_ATTR_TYPE_CONFED_SEQ:
 			as = ownAs
 		default:
-			return config.RPKI_VALIDATION_RESULT_TYPE_NOT_FOUND
+			return config.RPKI_VALIDATION_RESULT_TYPE_NOT_FOUND, nil
 		}
 	}
 	_, n, _ := net.ParseCIDR(cidr)
 	ones, _ := n.Mask.Size()
 	prefixLen := uint8(ones)
-	_, b, _ := tree.LongestPrefix(table.IpToRadixkey(n.IP, prefixLen))
+	key := table.IpToRadixkey(n.IP, prefixLen)
+	_, b, _ := tree.LongestPrefix(key)
 	if b == nil {
-		return config.RPKI_VALIDATION_RESULT_TYPE_NOT_FOUND
+		return config.RPKI_VALIDATION_RESULT_TYPE_NOT_FOUND, nil
 	}
 
-	bucket, _ := b.(*roaBucket)
-	for _, r := range bucket.entries {
-		if prefixLen > r.MaxLen {
-			continue
+	result := config.RPKI_VALIDATION_RESULT_TYPE_INVALID
+	var bucket *RoaBucket
+	fn := radix.WalkFn(func(k string, v interface{}) bool {
+		bucket, _ = v.(*RoaBucket)
+		for _, r := range bucket.entries {
+			if prefixLen <= r.MaxLen && r.AS != 0 && r.AS == as {
+				result = config.RPKI_VALIDATION_RESULT_TYPE_VALID
+				return true
+			}
 		}
-		if r.AS == as {
-			return config.RPKI_VALIDATION_RESULT_TYPE_VALID
-		}
-	}
-	return config.RPKI_VALIDATION_RESULT_TYPE_INVALID
+		return false
+	})
+	tree.WalkPath(key, fn)
+	return result, bucket
 }
 
 func (c *roaManager) validate(pathList []*table.Path) {
@@ -541,14 +545,13 @@ func (c *roaManager) validate(pathList []*table.Path) {
 			continue
 		}
 		if tree, ok := c.Roas[path.GetRouteFamily()]; ok {
-			r := validatePath(c.AS, tree, path.GetNlri().String(), path.GetAsPath())
+			r, _ := ValidatePath(c.AS, tree, path.GetNlri().String(), path.GetAsPath())
 			path.SetValidation(config.RpkiValidationResultType(r))
 		}
 	}
 }
 
 type roaClient struct {
-	t            tomb.Tomb
 	host         string
 	conn         *net.TCPConn
 	state        config.RpkiServerState
@@ -560,15 +563,22 @@ type roaClient struct {
 	lifetime     int64
 	endOfData    bool
 	pendingROAs  []*table.ROA
+	cancelfnc    context.CancelFunc
+	ctx          context.Context
 }
 
 func NewRoaClient(address, port string, ch chan *ROAEvent, lifetime int64) *roaClient {
-	return &roaClient{
+	ctx, cancel := context.WithCancel(context.Background())
+	c := &roaClient{
 		host:        net.JoinHostPort(address, port),
 		eventCh:     ch,
 		lifetime:    lifetime,
 		pendingROAs: make([]*table.ROA, 0),
+		ctx:         ctx,
+		cancelfnc:   cancel,
 	}
+	go c.tryConnect()
+	return c
 }
 
 func (c *roaClient) enable(serial uint32) error {
@@ -600,60 +610,63 @@ func (c *roaClient) softReset() error {
 }
 
 func (c *roaClient) reset() {
-	c.t.Kill(nil)
 	if c.conn != nil {
 		c.conn.Close()
 	}
 }
 
-func (c *roaClient) tryConnect() error {
-	for c.t.Alive() {
-		conn, err := net.Dial("tcp", c.host)
-		if err != nil {
-			time.Sleep(30 * time.Second)
+func (c *roaClient) stop() {
+	c.cancelfnc()
+	c.reset()
+}
+
+func (c *roaClient) tryConnect() {
+	for {
+		select {
+		case <-c.ctx.Done():
+			return
+		default:
+		}
+		if conn, err := net.Dial("tcp", c.host); err != nil {
+			// better to use context with timeout
+			time.Sleep(CONNECT_RETRY_INTERVAL * time.Second)
 		} else {
 			c.eventCh <- &ROAEvent{
 				EventType: CONNECTED,
 				Src:       c.host,
 				conn:      conn.(*net.TCPConn),
 			}
-			return nil
+			return
 		}
 	}
-	return nil
 }
 
-func (c *roaClient) established() error {
-	defer c.conn.Close()
-
-	disconnected := func() {
+func (c *roaClient) established() (err error) {
+	defer func() {
+		c.conn.Close()
 		c.eventCh <- &ROAEvent{
 			EventType: DISCONNECTED,
 			Src:       c.host,
 		}
-	}
+	}()
 
-	err := c.softReset()
-	if err != nil {
-		disconnected()
-		return nil
+	if err := c.softReset(); err != nil {
+		return err
 	}
 
 	for {
 		header := make([]byte, rtr.RTR_MIN_LEN)
-		_, err := io.ReadFull(c.conn, header)
-		if err != nil {
-			break
+		if _, err = io.ReadFull(c.conn, header); err != nil {
+			return err
 		}
 		totalLen := binary.BigEndian.Uint32(header[4:8])
 		if totalLen < rtr.RTR_MIN_LEN {
-			break
+			return fmt.Errorf("too short header length %v", totalLen)
 		}
 
 		body := make([]byte, totalLen-rtr.RTR_MIN_LEN)
-		_, err = io.ReadFull(c.conn, body)
-		if err != nil {
-			break
+		if _, err = io.ReadFull(c.conn, body); err != nil {
+			return
 		}
 
 		c.eventCh <- &ROAEvent{
@@ -661,8 +674,5 @@ func (c *roaClient) established() error {
 			Src:       c.host,
 			Data:      append(header, body...),
 		}
-
 	}
-	disconnected()
-	return nil
 }
