@@ -23,9 +23,9 @@ import (
 	"net"
 	"sort"
 
-	log "github.com/Sirupsen/logrus"
 	"github.com/osrg/gobgp/config"
 	"github.com/osrg/gobgp/packet/bgp"
+	log "github.com/sirupsen/logrus"
 )
 
 var SelectionOptions config.RouteSelectionOptionsConfig
@@ -53,7 +53,7 @@ const (
 func IpToRadixkey(b []byte, max uint8) string {
 	var buffer bytes.Buffer
 	for i := 0; i < len(b) && i < int(max); i++ {
-		buffer.WriteString(fmt.Sprintf("%08b", b[i]))
+		fmt.Fprintf(&buffer, "%08b", b[i])
 	}
 	return buffer.String()[:max]
 }
@@ -64,6 +64,24 @@ func CidrToRadixkey(cidr string) string {
 	return IpToRadixkey(n.IP, uint8(ones))
 }
 
+func AddrToRadixkey(addr bgp.AddrPrefixInterface) string {
+	var (
+		ip   net.IP
+		size uint8
+	)
+	switch T := addr.(type) {
+	case *bgp.IPAddrPrefix:
+		mask := net.CIDRMask(int(T.Length), net.IPv4len*8)
+		ip, size = T.Prefix.Mask(mask).To4(), uint8(T.Length)
+	case *bgp.IPv6AddrPrefix:
+		mask := net.CIDRMask(int(T.Length), net.IPv6len*8)
+		ip, size = T.Prefix.Mask(mask).To16(), uint8(T.Length)
+	default:
+		return CidrToRadixkey(addr.String())
+	}
+	return IpToRadixkey(ip, size)
+}
+
 type PeerInfo struct {
 	AS                      uint32
 	ID                      net.IP
@@ -108,12 +126,14 @@ func (i *PeerInfo) String() string {
 
 func NewPeerInfo(g *config.Global, p *config.Neighbor) *PeerInfo {
 	id := net.ParseIP(string(p.RouteReflector.Config.RouteReflectorClusterId)).To4()
+	// exclude zone info
+	naddr, _ := net.ResolveIPAddr("ip", p.State.NeighborAddress)
 	return &PeerInfo{
 		AS:                      p.Config.PeerAs,
 		LocalAS:                 g.Config.As,
 		LocalID:                 net.ParseIP(g.Config.RouterId).To4(),
-		Address:                 net.ParseIP(p.Config.NeighborAddress),
 		RouteReflectorClient:    p.RouteReflector.Config.RouteReflectorClient,
+		Address:                 naddr.IP,
 		RouteReflectorClusterID: id,
 		MultihopTtl:             p.EbgpMultihop.Config.MultihopTtl,
 	}
@@ -138,7 +158,7 @@ func NewDestination(nlri bgp.AddrPrefixInterface, known ...*Path) *Destination {
 	}
 	switch d.routeFamily {
 	case bgp.RF_IPv4_UC, bgp.RF_IPv6_UC, bgp.RF_IPv4_MPLS, bgp.RF_IPv6_MPLS:
-		d.RadixKey = CidrToRadixkey(nlri.String())
+		d.RadixKey = AddrToRadixkey(nlri)
 	}
 	return d
 }
@@ -173,15 +193,39 @@ func (dd *Destination) GetKnownPathList(id string) []*Path {
 	return list
 }
 
-func (dd *Destination) GetBestPath(id string) *Path {
-	for _, p := range dd.knownPathList {
-		if p.Filtered(id) == POLICY_DIRECTION_NONE {
+func getBestPath(id string, pathList *paths) *Path {
+	for _, p := range *pathList {
+		if p.Filtered(id) == POLICY_DIRECTION_NONE && !p.IsNexthopInvalid {
 			return p
 		}
 	}
 	return nil
 }
 
+func (dd *Destination) GetBestPath(id string) *Path {
+	return getBestPath(id, &dd.knownPathList)
+}
+
+func getMultiBestPath(id string, pathList *paths) []*Path {
+	list := make([]*Path, 0, len(*pathList))
+	var best *Path
+	for _, p := range *pathList {
+		if p.Filtered(id) == POLICY_DIRECTION_NONE && !p.IsNexthopInvalid {
+			if best == nil {
+				best = p
+				list = append(list, p)
+			} else if best.Compare(p) == 0 {
+				list = append(list, p)
+			}
+		}
+	}
+	return list
+}
+
+func (dd *Destination) GetMultiBestPath(id string) []*Path {
+	return getMultiBestPath(id, &dd.knownPathList)
+}
+
 func (dd *Destination) AddWithdraw(withdraw *Path) {
 	dd.validatePath(withdraw)
 	dd.withdrawList = append(dd.withdrawList, withdraw)
@@ -208,7 +252,7 @@ func (dd *Destination) validatePath(path *Path) {
 //
 // Modifies destination's state related to stored paths. Removes withdrawn
 // paths from known paths. Also, adds new paths to known paths.
-func (dest *Destination) Calculate(ids []string) (map[string]*Path, map[string]*Path, []*Path) {
+func (dest *Destination) Calculate(ids []string, peerDown bool) (map[string]*Path, map[string]*Path, []*Path) {
 	bestList := make(map[string]*Path, len(ids))
 	oldList := make(map[string]*Path, len(ids))
 	oldKnownPathList := dest.knownPathList
@@ -224,14 +268,7 @@ func (dest *Destination) Calculate(ids []string) (map[string]*Path, map[string]*
 	dest.computeKnownBestPath()
 
 	f := func(id string) (*Path, *Path) {
-		old := func() *Path {
-			for _, p := range oldKnownPathList {
-				if p.Filtered(id) == POLICY_DIRECTION_NONE {
-					return p
-				}
-			}
-			return nil
-		}()
+		old := getBestPath(id, &oldKnownPathList)
 		best := dest.GetBestPath(id)
 		if best != nil && best.Equal(old) {
 			// RFC4684 3.2. Intra-AS VPN Route Distribution
@@ -242,12 +279,24 @@ func (dest *Destination) Calculate(ids []string) (map[string]*Path, map[string]*
 			if best.GetRouteFamily() == bgp.RF_RTC_UC {
 				return best, old
 			}
+			// For BGP Nexthop Tracking, checks if the nexthop reachability
+			// was changed or not.
+			if best.IsNexthopInvalid != old.IsNexthopInvalid {
+				return best, old
+			}
 			return nil, old
 		}
 		if best == nil {
 			if old == nil {
 				return nil, nil
 			}
+			if peerDown {
+				// withdraws were generated by peer
+				// down so paths are not in knowpath
+				// or adjin.
+				old.IsWithdraw = true
+				return old, old
+			}
 			return old.Clone(true), old
 		}
 		return best, old
@@ -257,21 +306,6 @@ func (dest *Destination) Calculate(ids []string) (map[string]*Path, map[string]*
 	for _, id := range ids {
 		bestList[id], oldList[id] = f(id)
 		if id == GLOBAL_RIB_NAME && UseMultiplePaths.Enabled {
-			multipath := func(paths []*Path) []*Path {
-				mp := make([]*Path, 0, len(paths))
-				var best *Path
-				for _, path := range paths {
-					if path.Filtered(id) == POLICY_DIRECTION_NONE {
-						if best == nil {
-							best = path
-							mp = append(mp, path)
-						} else if best.Compare(path) == 0 {
-							mp = append(mp, path)
-						}
-					}
-				}
-				return mp
-			}
 			diff := func(lhs, rhs []*Path) bool {
 				if len(lhs) != len(rhs) {
 					return true
@@ -283,8 +317,8 @@ func (dest *Destination) Calculate(ids []string) (map[string]*Path, map[string]*
 				}
 				return false
 			}
-			oldM := multipath(oldKnownPathList)
-			newM := multipath(dest.knownPathList)
+			oldM := getMultiBestPath(id, &oldKnownPathList)
+			newM := dest.GetMultiBestPath(id)
 			if diff(oldM, newM) {
 				multi = newM
 				if len(newM) == 0 {
@@ -428,10 +462,20 @@ func (dest *Destination) computeKnownBestPath() (*Path, BestPathReason, error) {
 	// tie between two new paths learned in one cycle for which best-path
 	// calculation steps lead to tie.
 	if len(dest.knownPathList) == 1 {
+		// If the first path has the invalidated next-hop, which evaluated by
+		// IGP, returns no path with the reason of the next-hop reachability.
+		if dest.knownPathList[0].IsNexthopInvalid {
+			return nil, BPR_REACHABLE_NEXT_HOP, nil
+		}
 		return dest.knownPathList[0], BPR_ONLY_PATH, nil
 	}
 	sort.Sort(dest.knownPathList)
 	newBest := dest.knownPathList[0]
+	// If the first path has the invalidated next-hop, which evaluated by IGP,
+	// returns no path with the reason of the next-hop reachability.
+	if dest.knownPathList[0].IsNexthopInvalid {
+		return nil, BPR_REACHABLE_NEXT_HOP, nil
+	}
 	return newBest, newBest.reason, nil
 }
 
@@ -568,11 +612,18 @@ func compareByLLGRStaleCommunity(path1, path2 *Path) *Path {
 func compareByReachableNexthop(path1, path2 *Path) *Path {
 	//	Compares given paths and selects best path based on reachable next-hop.
 	//
-	//	If no path matches this criteria, return None.
-	//  However RouteServer doesn't need to check reachability, so return nil.
+	//	If no path matches this criteria, return nil.
+	//	For BGP Nexthop Tracking, evaluates next-hop is validated by IGP.
 	log.WithFields(log.Fields{
 		"Topic": "Table",
 	}).Debugf("enter compareByReachableNexthop -- path1: %s, path2: %s", path1, path2)
+
+	if path1.IsNexthopInvalid && !path2.IsNexthopInvalid {
+		return path2
+	} else if !path1.IsNexthopInvalid && path2.IsNexthopInvalid {
+		return path1
+	}
+
 	return nil
 }
 
