@@ -13,17 +13,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from fabric.api import local, lcd
-from fabric import colors
-from fabric.utils import indent
-from fabric.state import env, output
-from docker import Client
+from __future__ import absolute_import
 
-import netaddr
 import os
 import time
 import itertools
 
+from fabric.api import local, lcd
+from fabric import colors
+from fabric.state import env, output
+try:
+    from docker import Client
+except ImportError:
+    from docker import APIClient as Client
+import netaddr
+
 DEFAULT_TEST_PREFIX = ''
 DEFAULT_TEST_BASE_DIR = '/tmp/gobgp'
 TEST_PREFIX = DEFAULT_TEST_PREFIX
@@ -47,6 +51,7 @@ BGP_ATTR_TYPE_EXTENDED_COMMUNITIES = 16
 env.abort_exception = RuntimeError
 output.stderr = False
 
+
 def wait_for_completion(f, timeout=120):
     interval = 1
     count = 0
@@ -61,8 +66,8 @@ def wait_for_completion(f, timeout=120):
 
 
 def try_several_times(f, t=3, s=1):
-    e = None
-    for i in range(t):
+    e = Exception
+    for _ in range(t):
         try:
             r = f()
         except RuntimeError as e:
@@ -73,11 +78,11 @@ def try_several_times(f, t=3, s=1):
 
 
 def get_bridges():
-    return try_several_times(lambda : local("docker network ls | awk 'NR > 1{print $2}'", capture=True)).split('\n')
+    return try_several_times(lambda: local("docker network ls | awk 'NR > 1{print $2}'", capture=True)).split('\n')
 
 
 def get_containers():
-    return try_several_times(lambda : local("docker ps -a | awk 'NR > 1 {print $NF}'", capture=True)).split('\n')
+    return try_several_times(lambda: local("docker ps -a | awk 'NR > 1 {print $NF}'", capture=True)).split('\n')
 
 
 class CmdBuffer(list):
@@ -98,11 +103,10 @@ def make_gobgp_ctn(tag='gobgp', local_gobgp_path='', from_image='osrg/quagga'):
 
     c = CmdBuffer()
     c << 'FROM {0}'.format(from_image)
+    c << 'RUN go get -d github.com/osrg/gobgp/...; exit 0'
+    c << 'RUN rm -rf /go/src/github.com/osrg/gobgp'
     c << 'ADD gobgp /go/src/github.com/osrg/gobgp/'
-    c << 'RUN go get github.com/osrg/gobgp/gobgpd'
-    c << 'RUN go install github.com/osrg/gobgp/gobgpd'
-    c << 'RUN go get github.com/osrg/gobgp/gobgp'
-    c << 'RUN go install github.com/osrg/gobgp/gobgp'
+    c << 'RUN go get github.com/osrg/gobgp/...'
 
     rindex = local_gobgp_path.rindex('gobgp')
     if rindex < 0:
@@ -124,10 +128,10 @@ class Bridge(object):
         if with_ip:
             self.subnet = netaddr.IPNetwork(subnet)
 
-            def f():
+            def _f():
                 for host in self.subnet:
                     yield host
-            self._ip_generator = f()
+            self._ip_generator = _f()
             # throw away first network address
             self.next_ip_address()
 
@@ -143,7 +147,7 @@ class Bridge(object):
         self.self_ip = self_ip
         if self_ip:
             self.ip_addr = self.next_ip_address()
-            try_several_times(lambda :local("ip addr add {0} dev {1}".format(self.ip_addr, self.name)))
+            try_several_times(lambda: local("ip addr add {0} dev {1}".format(self.ip_addr, self.name)))
         self.ctns = []
 
     def next_ip_address(self):
@@ -151,7 +155,7 @@ class Bridge(object):
                                 self.subnet.prefixlen)
 
     def addif(self, ctn):
-        name = ctn.next_if_name()
+        _name = ctn.next_if_name()
         self.ctns.append(ctn)
         local("docker network connect {0} {1}".format(self.name, ctn.docker_name()))
         i = [x for x in Client(timeout=60, version='auto').inspect_network(self.id)['Containers'].values() if x['Name'] == ctn.docker_name()][0]
@@ -162,7 +166,7 @@ class Bridge(object):
         ctn.ip_addrs.append(('eth1', addr, self.name))
 
     def delete(self):
-        try_several_times(lambda : local("docker network rm {0}".format(self.name)))
+        try_several_times(lambda: local("docker network rm {0}".format(self.name)))
 
 
 class Container(object):
@@ -185,7 +189,7 @@ class Container(object):
         return '{0}_{1}'.format(TEST_PREFIX, self.name)
 
     def next_if_name(self):
-        name = 'eth{0}'.format(len(self.eths)+1)
+        name = 'eth{0}'.format(len(self.eths) + 1)
         self.eths.append(name)
         return name
 
@@ -195,7 +199,7 @@ class Container(object):
         for sv in self.shared_volumes:
             c << "-v {0}:{1}".format(sv[0], sv[1])
         c << "--name {0} -id {1}".format(self.docker_name(), self.image)
-        self.id = try_several_times(lambda : local(str(c), capture=True))
+        self.id = try_several_times(lambda: local(str(c), capture=True))
         self.is_running = True
         self.local("ip li set up dev lo")
         for line in self.local("ip a show dev eth0", capture=True).split('\n'):
@@ -208,12 +212,12 @@ class Container(object):
         return 0
 
     def stop(self):
-        ret = try_several_times(lambda : local("docker stop -t 0 " + self.docker_name(), capture=True))
+        ret = try_several_times(lambda: local("docker stop -t 0 " + self.docker_name(), capture=True))
         self.is_running = False
         return ret
 
     def remove(self):
-        ret = try_several_times(lambda : local("docker rm -f " + self.docker_name(), capture=True))
+        ret = try_several_times(lambda: local("docker rm -f " + self.docker_name(), capture=True))
         self.is_running = False
         return ret
 
@@ -230,7 +234,7 @@ class Container(object):
             intf_name = "eth1"
         c << "{0} {1}".format(self.docker_name(), ip_addr)
         self.ip_addrs.append((intf_name, ip_addr, bridge.name))
-        try_several_times(lambda :local(str(c)))
+        try_several_times(lambda: local(str(c)))
 
     def local(self, cmd, capture=False, stream=False, detach=False, tty=True):
         if stream:
@@ -270,7 +274,7 @@ class BGPContainer(Container):
 
     def __init__(self, name, asn, router_id, ctn_image_name):
         self.config_dir = '/'.join((TEST_BASE_DIR, TEST_PREFIX, name))
-        local('if [ -e {0} ]; then rm -r {0}; fi'.format(self.config_dir))
+        local('if [ -e {0} ]; then rm -rf {0}; fi'.format(self.config_dir))
         local('mkdir -p {0}'.format(self.config_dir))
         local('chmod 777 {0}'.format(self.config_dir))
         self.asn = asn
@@ -281,43 +285,59 @@ class BGPContainer(Container):
         super(BGPContainer, self).__init__(name, ctn_image_name)
 
     def __repr__(self):
-        return str({'name':self.name, 'asn':self.asn, 'router_id':self.router_id})
+        return str({'name': self.name, 'asn': self.asn, 'router_id': self.router_id})
 
     def run(self):
         self.create_config()
         super(BGPContainer, self).run()
         return self.WAIT_FOR_BOOT
 
+    def peer_name(self, peer):
+        if peer not in self.peers:
+            raise Exception('not found peer {0}'.format(peer.router_id))
+        name = self.peers[peer]['interface']
+        if name == '':
+            name = self.peers[peer]['neigh_addr'].split('/')[0]
+        return name
+
+    def update_peer(self, peer, **kwargs):
+        if peer not in self.peers:
+            raise Exception('peer not exists')
+        self.add_peer(peer, **kwargs)
+
     def add_peer(self, peer, passwd=None, vpn=False, is_rs_client=False,
                  policies=None, passive=False,
                  is_rr_client=False, cluster_id=None,
                  flowspec=False, bridge='', reload_config=True, as2=False,
                  graceful_restart=None, local_as=None, prefix_limit=None,
-                 v6=False, llgr=None, vrf=''):
+                 v6=False, llgr=None, vrf='', interface='', allow_as_in=0,
+                 remove_private_as=None, replace_peer_as=False):
         neigh_addr = ''
         local_addr = ''
         it = itertools.product(self.ip_addrs, peer.ip_addrs)
         if v6:
             it = itertools.product(self.ip6_addrs, peer.ip6_addrs)
 
-        for me, you in it:
-            if bridge != '' and bridge != me[2]:
-                continue
-            if me[2] == you[2]:
-                neigh_addr = you[1]
-                local_addr = me[1]
-                if v6:
-                    addr, mask = local_addr.split('/')
-                    local_addr = "{0}%{1}/{2}".format(addr, me[0], mask)
-                break
+        if interface == '':
+            for me, you in it:
+                if bridge != '' and bridge != me[2]:
+                    continue
+                if me[2] == you[2]:
+                    neigh_addr = you[1]
+                    local_addr = me[1]
+                    if v6:
+                        addr, mask = local_addr.split('/')
+                        local_addr = "{0}%{1}/{2}".format(addr, me[0], mask)
+                    break
 
-        if neigh_addr == '':
-            raise Exception('peer {0} seems not ip reachable'.format(peer))
+            if neigh_addr == '':
+                raise Exception('peer {0} seems not ip reachable'.format(peer))
 
         if not policies:
             policies = {}
 
         self.peers[peer] = {'neigh_addr': neigh_addr,
+                            'interface': interface,
                             'passwd': passwd,
                             'vpn': vpn,
                             'flowspec': flowspec,
@@ -332,7 +352,10 @@ class BGPContainer(Container):
                             'local_as': local_as,
                             'prefix_limit': prefix_limit,
                             'llgr': llgr,
-                            'vrf': vrf}
+                            'vrf': vrf,
+                            'allow_as_in': allow_as_in,
+                            'remove_private_as': remove_private_as,
+                            'replace_peer_as': replace_peer_as}
         if self.is_running and reload_config:
             self.create_config()
             self.reload_config()
@@ -366,7 +389,7 @@ class BGPContainer(Container):
                               'local-pref': local_pref,
                               'extended-community': extendedcommunity,
                               'matchs': matchs,
-                              'thens' : thens}
+                              'thens': thens}
         if self.is_running and reload_config:
             self.create_config()
             self.reload_config()
@@ -408,27 +431,27 @@ class BGPContainer(Container):
         raise Exception('implement get_neighbor() method')
 
     def get_reachablily(self, prefix, timeout=20):
-            version = netaddr.IPNetwork(prefix).version
-            addr = prefix.split('/')[0]
-            if version == 4:
-                ping_cmd = 'ping'
-            elif version == 6:
-                ping_cmd = 'ping6'
-            else:
-                raise Exception('unsupported route family: {0}'.format(version))
-            cmd = '/bin/bash -c "/bin/{0} -c 1 -w 1 {1} | xargs echo"'.format(ping_cmd, addr)
-            interval = 1
-            count = 0
-            while True:
-                res = self.local(cmd, capture=True)
-                print colors.yellow(res)
-                if '1 packets received' in res and '0% packet loss':
-                    break
-                time.sleep(interval)
-                count += interval
-                if count >= timeout:
-                    raise Exception('timeout')
-            return True
+        version = netaddr.IPNetwork(prefix).version
+        addr = prefix.split('/')[0]
+        if version == 4:
+            ping_cmd = 'ping'
+        elif version == 6:
+            ping_cmd = 'ping6'
+        else:
+            raise Exception('unsupported route family: {0}'.format(version))
+        cmd = '/bin/bash -c "/bin/{0} -c 1 -w 1 {1} | xargs echo"'.format(ping_cmd, addr)
+        interval = 1
+        count = 0
+        while True:
+            res = self.local(cmd, capture=True)
+            print colors.yellow(res)
+            if '1 packets received' in res and '0% packet loss':
+                break
+            time.sleep(interval)
+            count += interval
+            if count >= timeout:
+                raise Exception('timeout')
+        return True
 
     def wait_for(self, expected_state, peer, timeout=120):
         interval = 1
@@ -460,3 +483,31 @@ class BGPContainer(Container):
 
     def reload_config(self):
         raise Exception('implement reload_config() method')
+
+
+class OSPFContainer(Container):
+    WAIT_FOR_BOOT = 1
+
+    def __init__(self, name, ctn_image_name):
+        self.config_dir = '/'.join((TEST_BASE_DIR, TEST_PREFIX, name))
+        local('if [ -e {0} ]; then rm -rf {0}; fi'.format(self.config_dir))
+        local('mkdir -p {0}'.format(self.config_dir))
+        local('chmod 777 {0}'.format(self.config_dir))
+
+        # Example:
+        # networks = {
+        #     '192.168.1.0/24': '0.0.0.0',  # <network>: <area>
+        # }
+        self.networks = {}
+        super(OSPFContainer, self).__init__(name, ctn_image_name)
+
+    def __repr__(self):
+        return str({'name': self.name, 'networks': self.networks})
+
+    def run(self):
+        self.create_config()
+        super(OSPFContainer, self).run()
+        return self.WAIT_FOR_BOOT
+
+    def create_config(self):
+        raise NotImplementedError
