@@ -23,7 +23,7 @@ import (
 	"strings"
 	"syscall"
 
-	log "github.com/Sirupsen/logrus"
+	log "github.com/sirupsen/logrus"
 )
 
 const (
@@ -159,6 +159,12 @@ const (
 	ROUTER_ID_DELETE
 	ROUTER_ID_UPDATE
 	HELLO
+	IPV4_NEXTHOP_LOOKUP_MRIB
+	VRF_UNREGISTER
+	INTERFACE_LINK_PARAMS
+	NEXTHOP_REGISTER
+	NEXTHOP_UNREGISTER
+	NEXTHOP_UPDATE
 	MESSAGE_MAX
 )
 
@@ -211,6 +217,7 @@ const (
 	MESSAGE_IFINDEX  = 0x02
 	MESSAGE_DISTANCE = 0x04
 	MESSAGE_METRIC   = 0x08
+	MESSAGE_MTU      = 0x10
 )
 
 // Message Flags
@@ -325,46 +332,71 @@ func NewClient(network, address string, typ ROUTE_TYPE, version uint8) (*Client,
 		}
 	}()
 
+	// Send HELLO/ROUTER_ID_ADD messages to negotiate the Zebra message version.
+	c.SendHello()
+	c.SendRouterIDAdd()
+
+	receiveSingleMsg := func() (*Message, error) {
+		headerBuf, err := readAll(conn, int(HeaderSize(version)))
+		if err != nil {
+			err = fmt.Errorf("failed to read header: %s", err)
+			log.WithFields(log.Fields{
+				"Topic": "Zebra",
+			}).Error(err)
+			return nil, err
+		}
+		log.WithFields(log.Fields{
+			"Topic": "Zebra",
+		}).Debugf("read header from zebra: %v", headerBuf)
+		hd := &Header{}
+		err = hd.DecodeFromBytes(headerBuf)
+		if err != nil {
+			err = fmt.Errorf("failed to decode header: %s", err)
+			log.WithFields(log.Fields{
+				"Topic": "Zebra",
+			}).Error(err)
+			return nil, err
+		}
+
+		bodyBuf, err := readAll(conn, int(hd.Len-HeaderSize(version)))
+		if err != nil {
+			err = fmt.Errorf("failed to read body: %s", err)
+			log.WithFields(log.Fields{
+				"Topic": "Zebra",
+			}).Error(err)
+			return nil, err
+		}
+		log.WithFields(log.Fields{
+			"Topic": "Zebra",
+		}).Debugf("read body from zebra: %v", bodyBuf)
+		m, err := ParseMessage(hd, bodyBuf)
+		if err != nil {
+			log.WithFields(log.Fields{
+				"Topic": "Zebra",
+			}).Warnf("failed to parse message: %s", err)
+			return nil, nil
+		}
+
+		return m, nil
+	}
+
+	// Try to receive the first message from Zebra.
+	if m, err := receiveSingleMsg(); err != nil {
+		c.Close()
+		// Return error explicitly in order to retry connection.
+		return nil, err
+	} else if m != nil {
+		incoming <- m
+	}
+
+	// Start receive loop only when the first message successfully received.
 	go func() {
 		for {
-			headerBuf, err := readAll(conn, int(HeaderSize(version)))
-			if err != nil {
-				log.WithFields(log.Fields{
-					"Topic": "Zebra",
-				}).Errorf("failed to read header:%s", err)
+			if m, err := receiveSingleMsg(); err != nil {
 				return
+			} else if m != nil {
+				incoming <- m
 			}
-			log.WithFields(log.Fields{
-				"Topic": "Zebra",
-			}).Debugf("read header from zebra: %v", headerBuf)
-			hd := &Header{}
-			err = hd.DecodeFromBytes(headerBuf)
-			if err != nil {
-				log.WithFields(log.Fields{
-					"Topic": "Zebra",
-				}).Errorf("failed to decode header:%s", err)
-				return
-			}
-
-			bodyBuf, err := readAll(conn, int(hd.Len-HeaderSize(version)))
-			if err != nil {
-				log.WithFields(log.Fields{
-					"Topic": "Zebra",
-				}).Errorf("failed to read body:%s", err)
-				return
-			}
-			log.WithFields(log.Fields{
-				"Topic": "Zebra",
-			}).Debugf("read body from zebra: %v", bodyBuf)
-			m, err := ParseMessage(hd, bodyBuf)
-			if err != nil {
-				log.WithFields(log.Fields{
-					"Topic": "Zebra",
-				}).Warnf("failed to parse message:%s", err)
-				continue
-			}
-
-			incoming <- m
 		}
 	}()
 
@@ -507,6 +539,7 @@ func (h *Header) DecodeFromBytes(data []byte) error {
 type Body interface {
 	DecodeFromBytes([]byte, uint8) error
 	Serialize() ([]byte, error)
+	String() string
 }
 
 type HelloBody struct {
@@ -522,6 +555,10 @@ func (b *HelloBody) Serialize() ([]byte, error) {
 	return []byte{uint8(b.RedistDefault)}, nil
 }
 
+func (b *HelloBody) String() string {
+	return fmt.Sprintf("route_type: %d", b.RedistDefault)
+}
+
 type RedistributeBody struct {
 	Redist ROUTE_TYPE
 }
@@ -535,6 +572,10 @@ func (b *RedistributeBody) Serialize() ([]byte, error) {
 	return []byte{uint8(b.Redist)}, nil
 }
 
+func (b *RedistributeBody) String() string {
+	return fmt.Sprintf("route_type: %d", b.Redist)
+}
+
 type InterfaceUpdateBody struct {
 	Name         string
 	Index        uint32
@@ -662,6 +703,7 @@ type IPRouteBody struct {
 	Ifindexs     []uint32
 	Distance     uint8
 	Metric       uint32
+	Mtu          uint32
 	Api          API_TYPE
 }
 
@@ -716,6 +758,11 @@ func (b *IPRouteBody) Serialize() ([]byte, error) {
 		binary.BigEndian.PutUint32(bbuf, b.Metric)
 		buf = append(buf, bbuf...)
 	}
+	if b.Message&MESSAGE_MTU > 0 {
+		bbuf := make([]byte, 4)
+		binary.BigEndian.PutUint32(bbuf, b.Mtu)
+		buf = append(buf, bbuf...)
+	}
 	return buf, nil
 }
 
@@ -769,6 +816,11 @@ func (b *IPRouteBody) DecodeFromBytes(data []byte, version uint8) error {
 		rest += 4
 	}
 
+	if b.Message&MESSAGE_MTU > 0 {
+		// mtu(4)
+		rest += 4
+	}
+
 	if len(data[pos:]) != rest {
 		return fmt.Errorf("message length invalid")
 	}
@@ -798,18 +850,23 @@ func (b *IPRouteBody) DecodeFromBytes(data []byte, version uint8) error {
 
 	if b.Message&MESSAGE_DISTANCE > 0 {
 		b.Distance = data[pos]
+		pos += 1
 	}
 	if b.Message&MESSAGE_METRIC > 0 {
-		pos += 1
 		b.Metric = binary.BigEndian.Uint32(data[pos : pos+4])
+		pos += 4
+	}
+	if b.Message&MESSAGE_MTU > 0 {
+		b.Mtu = binary.BigEndian.Uint32(data[pos : pos+4])
+		pos += 4
 	}
 
 	return nil
 }
 
 func (b *IPRouteBody) String() string {
-	s := fmt.Sprintf("type: %s, flags: %s, message: %d, prefix: %s, length: %d, nexthop: %s, distance: %d, metric: %d",
-		b.Type.String(), b.Flags.String(), b.Message, b.Prefix.String(), b.PrefixLength, b.Nexthops[0].String(), b.Distance, b.Metric)
+	s := fmt.Sprintf("type: %s, flags: %s, message: %d, prefix: %s, length: %d, nexthop: %s, distance: %d, metric: %d, mtu: %d",
+		b.Type.String(), b.Flags.String(), b.Message, b.Prefix.String(), b.PrefixLength, b.Nexthops[0].String(), b.Distance, b.Metric, b.Mtu)
 	return s
 }
 
@@ -832,6 +889,87 @@ func (n *Nexthop) String() string {
 	return s
 }
 
+func serializeNexthops(nexthops []*Nexthop, isV4 bool) ([]byte, error) {
+	buf := make([]byte, 0)
+	if len(nexthops) == 0 {
+		return buf, nil
+	}
+	buf = append(buf, byte(len(nexthops)))
+
+	for _, nh := range nexthops {
+		buf = append(buf, byte(nh.Type))
+
+		switch nh.Type {
+		case NEXTHOP_IFINDEX, NEXTHOP_IFNAME:
+			bbuf := make([]byte, 4)
+			binary.BigEndian.PutUint32(bbuf, nh.Ifindex)
+			buf = append(buf, bbuf...)
+
+		case NEXTHOP_IPV4, NEXTHOP_IPV6:
+			if isV4 {
+				buf = append(buf, nh.Addr.To4()...)
+			} else {
+				buf = append(buf, nh.Addr.To16()...)
+			}
+
+		case NEXTHOP_IPV4_IFINDEX, NEXTHOP_IPV4_IFNAME, NEXTHOP_IPV6_IFINDEX, NEXTHOP_IPV6_IFNAME:
+			if isV4 {
+				buf = append(buf, nh.Addr.To4()...)
+			} else {
+				buf = append(buf, nh.Addr.To16()...)
+			}
+			bbuf := make([]byte, 4)
+			binary.BigEndian.PutUint32(bbuf, nh.Ifindex)
+			buf = append(buf, bbuf...)
+		}
+	}
+
+	return buf, nil
+}
+
+func decodeNexthopsFromBytes(nexthops *[]*Nexthop, data []byte, isV4 bool) (int, error) {
+	addrLen := net.IPv4len
+	if !isV4 {
+		addrLen = net.IPv6len
+	}
+
+	numNexthop := int(data[0])
+	offset := 1
+
+	for i := 0; i < numNexthop; i++ {
+		nh := &Nexthop{}
+		nh.Type = NEXTHOP_FLAG(data[offset])
+		offset += 1
+
+		switch nh.Type {
+		case NEXTHOP_IFINDEX, NEXTHOP_IFNAME:
+			nh.Ifindex = binary.BigEndian.Uint32(data[offset : offset+4])
+			offset += 4
+
+		case NEXTHOP_IPV4, NEXTHOP_IPV6:
+			if isV4 {
+				nh.Addr = net.IP(data[offset : offset+addrLen]).To4()
+			} else {
+				nh.Addr = net.IP(data[offset : offset+addrLen]).To16()
+			}
+			offset += addrLen
+
+		case NEXTHOP_IPV4_IFINDEX, NEXTHOP_IPV4_IFNAME, NEXTHOP_IPV6_IFINDEX, NEXTHOP_IPV6_IFNAME:
+			if isV4 {
+				nh.Addr = net.IP(data[offset : offset+addrLen]).To4()
+			} else {
+				nh.Addr = net.IP(data[offset : offset+addrLen]).To16()
+			}
+			offset += addrLen
+			nh.Ifindex = binary.BigEndian.Uint32(data[offset : offset+4])
+			offset += 4
+		}
+		*nexthops = append(*nexthops, nh)
+	}
+
+	return offset, nil
+}
+
 func (b *NexthopLookupBody) Serialize() ([]byte, error) {
 
 	isV4 := b.Api == IPV4_NEXTHOP_LOOKUP
@@ -848,12 +986,12 @@ func (b *NexthopLookupBody) Serialize() ([]byte, error) {
 func (b *NexthopLookupBody) DecodeFromBytes(data []byte, version uint8) error {
 
 	isV4 := b.Api == IPV4_NEXTHOP_LOOKUP
-	var addrLen uint8 = net.IPv4len
+	addrLen := net.IPv4len
 	if !isV4 {
 		addrLen = net.IPv6len
 	}
 
-	if len(data) < int(addrLen) {
+	if len(data) < addrLen {
 		return fmt.Errorf("message length invalid")
 	}
 
@@ -870,45 +1008,11 @@ func (b *NexthopLookupBody) DecodeFromBytes(data []byte, version uint8) error {
 	if len(data[pos:]) > int(1+addrLen) {
 		b.Metric = binary.BigEndian.Uint32(data[pos : pos+4])
 		pos += 4
-		nexthopNum := int(data[pos])
 		b.Nexthops = []*Nexthop{}
-
-		if nexthopNum > 0 {
-			pos += 1
-			for i := 0; i < nexthopNum; i++ {
-				nh := &Nexthop{}
-				nh.Type = NEXTHOP_FLAG(data[pos])
-				pos += 1
-
-				switch nh.Type {
-				case NEXTHOP_IPV4, NEXTHOP_IPV6:
-					b := make([]byte, addrLen)
-					copy(b, data[pos:pos+addrLen])
-					if isV4 {
-						nh.Addr = net.IP(b).To4()
-					} else {
-						nh.Addr = net.IP(b).To16()
-					}
-					pos += addrLen
-
-				case NEXTHOP_IPV4_IFINDEX, NEXTHOP_IPV6_IFINDEX, NEXTHOP_IPV6_IFNAME:
-					b := make([]byte, addrLen)
-					copy(b, data[pos:pos+addrLen])
-					if isV4 {
-						nh.Addr = net.IP(b).To4()
-					} else {
-						nh.Addr = net.IP(b).To16()
-					}
-					pos += addrLen
-					nh.Ifindex = binary.BigEndian.Uint32(data[pos : pos+4])
-					pos += 4
-
-				case NEXTHOP_IFINDEX, NEXTHOP_IFNAME:
-					nh.Ifindex = binary.BigEndian.Uint32(data[pos : pos+4])
-					pos += 4
-				}
-				b.Nexthops = append(b.Nexthops, nh)
-			}
+		if nexthopsByteLen, err := decodeNexthopsFromBytes(&b.Nexthops, data[pos:], isV4); err != nil {
+			return err
+		} else {
+			pos += nexthopsByteLen
 		}
 	}
 
@@ -942,10 +1046,13 @@ func (b *ImportLookupBody) Serialize() ([]byte, error) {
 }
 
 func (b *ImportLookupBody) DecodeFromBytes(data []byte, version uint8) error {
+	isV4 := b.Api == IPV4_IMPORT_LOOKUP
+	addrLen := net.IPv4len
+	if !isV4 {
+		addrLen = net.IPv6len
+	}
 
-	var addrLen uint8 = net.IPv4len
-
-	if len(data) < int(addrLen) {
+	if len(data) < addrLen {
 		return fmt.Errorf("message length invalid")
 	}
 
@@ -958,36 +1065,11 @@ func (b *ImportLookupBody) DecodeFromBytes(data []byte, version uint8) error {
 	if len(data[pos:]) > int(1+addrLen) {
 		b.Metric = binary.BigEndian.Uint32(data[pos : pos+4])
 		pos += 4
-		nexthopNum := int(data[pos])
 		b.Nexthops = []*Nexthop{}
-		if nexthopNum > 0 {
-			pos += 1
-			for i := 0; i < nexthopNum; i++ {
-				nh := &Nexthop{}
-				nh.Type = NEXTHOP_FLAG(data[pos])
-				pos += 1
-
-				switch nh.Type {
-				case NEXTHOP_IPV4:
-					b := make([]byte, addrLen)
-					copy(b, data[pos:pos+addrLen])
-					nh.Addr = net.IP(b).To4()
-					pos += addrLen
-
-				case NEXTHOP_IPV4_IFINDEX:
-					b := make([]byte, addrLen)
-					copy(b, data[pos:pos+addrLen])
-					nh.Addr = net.IP(b).To4()
-					pos += addrLen
-					nh.Ifindex = binary.BigEndian.Uint32(data[pos : pos+4])
-					pos += 4
-
-				case NEXTHOP_IFINDEX, NEXTHOP_IFNAME:
-					nh.Ifindex = binary.BigEndian.Uint32(data[pos : pos+4])
-					pos += 4
-				}
-				b.Nexthops = append(b.Nexthops, nh)
-			}
+		if nexthopsByteLen, err := decodeNexthopsFromBytes(&b.Nexthops, data[pos:], isV4); err != nil {
+			return err
+		} else {
+			pos += nexthopsByteLen
 		}
 	}
 
@@ -1004,6 +1086,205 @@ func (b *ImportLookupBody) String() string {
 	return s
 }
 
+type RegisteredNexthop struct {
+	Connected uint8
+	Family    uint16
+	// Note: Ignores PrefixLength (uint8),
+	// because this field should be always:
+	// - 32 if Address Family is AF_INET
+	// - 128 if Address Family is AF_INET6
+	Prefix net.IP
+}
+
+func (n *RegisteredNexthop) Len() int {
+	// Connected (1 byte) + Address Family (2 bytes) + Prefix Length (1 byte) + Prefix (variable)
+	if n.Family == uint16(syscall.AF_INET) {
+		return 4 + net.IPv4len
+	} else {
+		return 4 + net.IPv6len
+	}
+}
+
+func (n *RegisteredNexthop) Serialize() ([]byte, error) {
+	// Connected (1 byte)
+	buf := make([]byte, 4)
+	buf[0] = byte(n.Connected)
+
+	// Address Family (2 bytes)
+	binary.BigEndian.PutUint16(buf[1:3], n.Family)
+
+	// Prefix Length (1 byte) + Prefix (variable)
+	switch n.Family {
+	case uint16(syscall.AF_INET):
+		buf[3] = byte(net.IPv4len * 8)
+		buf = append(buf, n.Prefix.To4()...)
+	case uint16(syscall.AF_INET6):
+		buf[3] = byte(net.IPv6len * 8)
+		buf = append(buf, n.Prefix.To16()...)
+	default:
+		return nil, fmt.Errorf("invalid address family: %d", n.Family)
+	}
+
+	return buf, nil
+}
+
+func (n *RegisteredNexthop) DecodeFromBytes(data []byte) error {
+	// Connected (1 byte)
+	n.Connected = uint8(data[0])
+	offset := 1
+
+	// Address Family (2 bytes)
+	n.Family = binary.BigEndian.Uint16(data[offset : offset+2])
+	isV4 := n.Family == uint16(syscall.AF_INET)
+	addrLen := int(net.IPv4len)
+	if !isV4 {
+		addrLen = net.IPv6len
+	}
+	// Note: Ignores Prefix Length (1 byte)
+	offset += 3
+
+	// Prefix (variable)
+	if isV4 {
+		n.Prefix = net.IP(data[offset : offset+addrLen]).To4()
+	} else {
+		n.Prefix = net.IP(data[offset : offset+addrLen]).To16()
+	}
+
+	return nil
+}
+
+func (n *RegisteredNexthop) String() string {
+	return fmt.Sprintf("connected: %d, family: %d, prefix: %s", n.Connected, n.Family, n.Prefix.String())
+}
+
+type NexthopRegisterBody struct {
+	Api      API_TYPE
+	Nexthops []*RegisteredNexthop
+}
+
+func (b *NexthopRegisterBody) Serialize() ([]byte, error) {
+	buf := make([]byte, 0)
+
+	// List of Registered Nexthops
+	for _, nh := range b.Nexthops {
+		nhBuf, err := nh.Serialize()
+		if err != nil {
+			return nil, err
+		}
+		buf = append(buf, nhBuf...)
+	}
+
+	return buf, nil
+}
+
+func (b *NexthopRegisterBody) DecodeFromBytes(data []byte, version uint8) error {
+	offset := 0
+
+	// List of Registered Nexthops
+	b.Nexthops = []*RegisteredNexthop{}
+	for len(data[offset:]) > 0 {
+		nh := new(RegisteredNexthop)
+		err := nh.DecodeFromBytes(data[offset:])
+		if err != nil {
+			return err
+		}
+		b.Nexthops = append(b.Nexthops, nh)
+
+		offset += nh.Len()
+		if len(data) < offset {
+			break
+		}
+	}
+
+	return nil
+}
+
+func (b *NexthopRegisterBody) String() string {
+	s := make([]string, 0)
+	for _, nh := range b.Nexthops {
+		s = append(s, fmt.Sprintf("nexthop:{%s}", nh.String()))
+	}
+	return strings.Join(s, ", ")
+}
+
+type NexthopUpdateBody struct {
+	Api    API_TYPE
+	Family uint16
+	// Note: Ignores PrefixLength (uint8),
+	// because this field should be always:
+	// - 32 if Address Family is AF_INET
+	// - 128 if Address Family is AF_INET6
+	Prefix   net.IP
+	Metric   uint32
+	Nexthops []*Nexthop
+}
+
+func (b *NexthopUpdateBody) Serialize() ([]byte, error) {
+	// Address Family (2 bytes)
+	buf := make([]byte, 3)
+	binary.BigEndian.PutUint16(buf, b.Family)
+
+	// Prefix Length (1 byte) + Prefix (variable)
+	switch b.Family {
+	case uint16(syscall.AF_INET):
+		buf[2] = byte(net.IPv4len * 8)
+		buf = append(buf, b.Prefix.To4()...)
+	case uint16(syscall.AF_INET6):
+		buf[2] = byte(net.IPv6len * 8)
+		buf = append(buf, b.Prefix.To16()...)
+	default:
+		return nil, fmt.Errorf("invalid address family: %d", b.Family)
+	}
+
+	return buf, nil
+}
+
+func (b *NexthopUpdateBody) DecodeFromBytes(data []byte, version uint8) error {
+	// Address Family (2 bytes)
+	b.Family = binary.BigEndian.Uint16(data[0:2])
+	isV4 := b.Family == uint16(syscall.AF_INET)
+	addrLen := int(net.IPv4len)
+	if !isV4 {
+		addrLen = net.IPv6len
+	}
+	// Note: Ignores Prefix Length (1 byte)
+	offset := 3
+
+	// Prefix (variable)
+	if isV4 {
+		b.Prefix = net.IP(data[offset : offset+addrLen]).To4()
+	} else {
+		b.Prefix = net.IP(data[offset : offset+addrLen]).To16()
+	}
+	offset += addrLen
+
+	// Metric (4 bytes)
+	// Number of Nexthops (1 byte)
+	if len(data[offset:]) < 4+1 {
+		return fmt.Errorf("invalid message length: missing metric(4 bytes) or nexthops(1 byte): %d<5", len(data[offset:]))
+	}
+	b.Metric = binary.BigEndian.Uint32(data[offset : offset+4])
+	offset += 4
+
+	// List of Nexthops
+	b.Nexthops = []*Nexthop{}
+	if nexthopsByteLen, err := decodeNexthopsFromBytes(&b.Nexthops, data[offset:], isV4); err != nil {
+		return err
+	} else {
+		offset += nexthopsByteLen
+	}
+
+	return nil
+}
+
+func (b *NexthopUpdateBody) String() string {
+	s := fmt.Sprintf("family: %d, prefix: %s, metric: %d", b.Family, b.Prefix.String(), b.Metric)
+	for _, nh := range b.Nexthops {
+		s = s + fmt.Sprintf(", nexthop:{%s}", nh.String())
+	}
+	return s
+}
+
 type Message struct {
 	Header Header
 	Body   Body
@@ -1051,6 +1332,11 @@ func ParseMessage(hdr *Header, data []byte) (*Message, error) {
 		log.WithFields(log.Fields{
 			"Topic": "Zebra",
 		}).Debugf("ipv4 import lookup message received: %v", data)
+	case NEXTHOP_UPDATE:
+		m.Body = &NexthopUpdateBody{Api: m.Header.Command}
+		log.WithFields(log.Fields{
+			"Topic": "Zebra",
+		}).Debugf("nexthop update message received: %v", data)
 	default:
 		return nil, fmt.Errorf("Unknown zapi command: %d", m.Header.Command)
 	}
